package edu.berkeley.sbp.misc;
import edu.berkeley.sbp.util.*;
import edu.berkeley.sbp.*;
import java.util.*;
import java.io.*;

public class MetaGrammar extends StringWalker {

    public static Union make() throws Exception {
        return ((MetaGrammar)new MetaGrammar().walk(meta)).done();
    }
    public String toString() {
        StringBuffer ret = new StringBuffer();
        for(Union u : nt.values()) u.toString(ret);
        return ret.toString();
    }

    public /*private*/ static HashSet<Element> dropAll = new HashSet<Element>();

    // Statics //////////////////////////////////////////////////////////////////////////////

    private static final Union SELF = new Union("()");

    ////////////////////////////////////////////////////////////////////////////////

    private Union g;
    private HashMap<String,Union> nt;
    private int anon = 0;
    private String startSymbol;
    private boolean strings;

    private Element  set(Range.Set r) { if (strings) throw new Error(); return CharToken.set(r); }
    private Element  string(String s) { return strings ? StringToken.string(s) : CharToken.string(s); }
    private Atom     leftBrace()      { return strings ? StringToken.leftBrace : CharToken.leftBrace; }
    private Atom     rightBrace()     { return strings ? StringToken.rightBrace : CharToken.rightBrace; }

    public MetaGrammar() { this("s", false); }
    public MetaGrammar(String s) { this(s, false); }
    public MetaGrammar(String s, boolean strings) { this.strings = strings; done(s); }
    public Union done() { return done("s"); }
    public Union done(String str) {
        Union ret = g;
        g = new Union(str);
        startSymbol = str;
        nt = new HashMap<String,Union>();
        dropAll = new HashSet<Element>();
        nt.put(str, g);
        return ret;
    }


    // MetaGrammar //////////////////////////////////////////////////////////////////////////////

    public Union       nonTerminal(String str) { return nonTerminal(str, null, false, false); }
    public Union       nonTerminal(String str, PreSequence[][] s, boolean synthetic, boolean dropAll) {
        Union n = str.equals(startSymbol) ? g : nt.get(str);
        if (n == null) nt.put(str, n = new Union(str, synthetic));
        if (dropAll) this.dropAll.add(n);
        if (s==null) return n;
        HashSet<Sequence> seqs = new HashSet<Sequence>();
        for(int i=0; i<s.length; i++) {
            if (s[i]==null) continue;
            HashSet<Sequence> temp = new HashSet<Sequence>();
            for(PreSequence pre : s[i]) {
                pre.not.addAll(seqs);
                Sequence seq = pre.buildSequence(n, false, dropAll);
                temp.add(seq);
                n.add(seq);
            }
            seqs.addAll(temp);
        }
        return n;
    }

    public String string(Iterable<Tree<String>> children) {
        String ret = "";
        for(Tree<String> t : children) ret += string(t);
        return ret;
    }
    public String string(Tree<String> tree) {
        String ret = "";
        if (tree.head()!=null) ret += tree.head();
        ret += string(tree.children());
        return ret;
    }

    public Object walk(Tree<String> tree) {
        String head = tree.head();
        if (tree.numChildren()==0) return super.walk(tree);
        if      ("\\n".equals(head)) return new Character('\n');
        else if ("\\r".equals(head)) return new Character('\r');
        else if ("grammar".equals(head)) { for(Tree<String> t : tree.children()) walk(t); return this; }
        else if ("*".equals(head))  return Repeat.many0((Element)walk(tree.child(0)));
        else if ("+".equals(head))  return Repeat.many1((Element)walk(tree.child(0)));
        else if ("+/".equals(head)) return Repeat.many1((Element)walk(tree.child(0)), (Element)walk(tree.child(1)));
        else if ("*/".equals(head)) return Repeat.many0((Element)walk(tree.child(0)), (Element)walk(tree.child(1)));
        else if ("**".equals(head)) return Repeat.maximal(Repeat.many0((Element)walk(tree.child(0))));
        else if ("++".equals(head)) return Repeat.maximal(Repeat.many1((Element)walk(tree.child(0))));
        else if ("?".equals(head))  return Repeat.maybe((Element)walk(tree.child(0)));
        else if ("&".equals(head))
            return ((PreSequence)walk(tree,0)).and(new PreSequence((Element[])Reflection.lub((Object[])walk(tree, 1)), null).buildSequence(null, true, false));
        else if ("&~".equals(head))
            return ((PreSequence)walk(tree,0)).not(new PreSequence((Element[])Reflection.lub((Object[])walk(tree, 1)), null).buildSequence(null, true, false));
        else if ("epsilon".equals(head)) return Union.epsilon;
        else if ("()".equals(head)) return Union.epsilon;
        else if (")".equals(head)) return SELF;
        else if ("nonTerminal".equals(head)) return nonTerminal(string(tree.child(0)), null, false, false);
        else if ("::=".equals(head)) return nonTerminal(string(tree.child(0)), (PreSequence[][])walk(tree, 1), false, false);
        else if ("!::=".equals(head)) return nonTerminal(string(tree.child(0)), (PreSequence[][])walk(tree, 1), false, true);
        else if ("(".equals(head)) return nonTerminal("anon"+(anon++), (PreSequence[][])walk(tree, 0), false, false);
        else if ("literal".equals(head)) { Element ret = string(string(tree.child(0))); dropAll.add(ret); return ret; }
        else if ("-".equals(head)) return new Range(walk(tree, 0).toString().charAt(0), walk(tree,1).toString().charAt(0));
        else if ("range".equals(head)) return new Range(walk(tree, 0).toString().charAt(0), walk(tree,0).toString().charAt(0));
        else if ("gram".equals(head)) return walk(tree, 1);
        else if ("=>".equals(head)) { PreSequence p = (PreSequence)walk(tree, 0); p.tag = string(tree.child(1)); return p; }
        else if ("/".equals(head)) return ((PreSequence)walk(tree, 0)).sparse((Element)walk(tree, 1));
        else if ("ps".equals(head)) return new PreSequence((Element[])walk(tree, 0));
        else if ("ps2".equals(head)) {
            Object[] o1 = (Object[])walk(tree, 0);
            String s = string(tree.child(1));
            Object[] o2 = (Object[])walk(tree, 2);
            if (o1==null) o1 = new Object[0];
            if (o2==null) o2 = new Object[0];
            Object[] o3 = new Object[o1.length + o2.length + 1];
            System.arraycopy(o1, 0, o3, 0, o1.length);
            o3[o1.length] = string(s);
            System.arraycopy(o2, 0, o3, o1.length+1, o2.length);
            PreSequence ret = new PreSequence(o3, s);
            ret.drops[o1.length] = o3.length>1;
            return ret;
        } else if ("[".equals(head) || "[~".equals(head)) {
            boolean positive = "[".equals(head);
            Range[] rr = (Range[])walk(tree, 0);
            Range.Set ret = positive ? new Range.Set() : new Range.Set(new Range(true, true));
            if (rr != null)
                for(Range r : rr)
                    if (positive) ret.add(r);
                    else          ret.remove(r);
            return set(ret);
        }
        else return super.walk(tree);
    }


    public Object walk(String tag, Object[] argo) {
        if (argo.length==0) return super.walk(tag, argo);
        if (argo==null) return tag;
        if (tag==null || "".equals(tag)) return argo;
        return super.walk(tag, argo);
    }

    //////////////////////////////////////////////////////////////////////////////

    public class PreSequence {
        public final HashSet<Sequence> and  = new HashSet<Sequence>();
        public final HashSet<Sequence> not  = new HashSet<Sequence>();
        public /*final*/ String tag;
        public final Object[] o;

        public PreSequence sparse(Object e) {
            Object[] ret;
            boolean[] drops;
            if (o.length <= 1) {
                ret = o;
                drops = new boolean[this.drops.length];
                System.arraycopy(this.drops, 0, drops, 0, this.drops.length);
            } else {
                ret = new Object[o.length * 2 - 1];
                drops = new boolean[o.length * 2 - 1];
                for(int i=0; i<o.length; i++) {
                    Object oi = o[i];
                    ret[i*2]   = oi;
                    drops[i*2] = this.drops[i];
                    if (i*2+1<ret.length) {
                        ret[i*2+1] = e;
                        drops[i*2+1] = true;
                    }
                }
            }
            PreSequence p = new PreSequence(ret, tag, drops);
            p.not.addAll(not);
            p.and.addAll(and);
            return p;
        }

        public PreSequence not(Sequence s) { not.add(s); return this; }
        public PreSequence and(Sequence s) { and.add(s); return this; }

        boolean[] drops = null;
        public PreSequence(Object o) { this(new Object[] { o }, null); }
        public PreSequence(Object[] o) { this(o, null); }
        public PreSequence(Object[] o, String tag) { this(o, tag, null); }
        public PreSequence(Object[] o, String tag, boolean[] drops) {
            this.o = o;
            this.tag = tag;
            this.drops = drops==null ? new boolean[o.length] : drops;
        }

        public Union    buildUnion() {
            Union u = new Union("???");
            u.add(buildSequence(u));
            return u;
        }
        public Sequence buildSequence(Union u) { return buildSequence(u, false, false); }
        public Sequence buildSequence(Union u, boolean lame, boolean dropAll) {
            for(Sequence s : and) u.add(s);
            for(Sequence s : not) u.add(s);
            HashSet<Sequence> set = new HashSet<Sequence>();
            Element[] o2 = o==null ? new Element[0] : new Element[o.length];
            int nonDrop = 0;
            if (o != null) {
                int j = 0;
                for(int i=0; i<o.length; i++) {
                    Object oi = o[i];
                    if (oi==SELF)                    o2[j] = u.new Subset("(("+u+"))", set);
                    else                             o2[j] = (Element)oi;

                    if (MetaGrammar.dropAll.contains(o2[j])) drops[j] = true;
                    nonDrop += drops[j] ? 0 : 1;
                    o2[j] = o2[j];
                    j++;
                }
            }
            Element[] expansion = o2;
            Sequence ret = null;
            if (dropAll || lame) ret = Sequence.drop(expansion, and, not, lame);
            else if (tag!=null) ret = Sequence.rewritingSequence(tag, expansion, drops, and, not);
            else {
                int idx = -1;
                for(int i=0; i<expansion.length; i++) {
                    if (!drops[i]) idx = i;
                    else expansion[i] = expansion[i];
                }
                ret = idx==-1 ? Sequence.drop(expansion, and, not, false) : Sequence.singleton(expansion, idx, and, not);
            }
            set.add(ret);
            return ret;
        }
    }

    public static void main(String[] args) throws Exception {
        if (args.length != 2) {
            System.err.println("usage: java " + MetaGrammar.class.getName() + " grammarfile.g com.yourdomain.package.ClassName");
            System.exit(-1);
        }

        String className   = args[1].substring(args[1].lastIndexOf('.')+1);
        String packageName = args[1].substring(0, args[1].lastIndexOf('.'));
        String fileName    = packageName.replace('.', '/') + "/" + className + ".java";

        BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(fileName)));
        StringBuffer out = new StringBuffer();

        boolean skip = false;
        for(String s = br.readLine(); s != null; s = br.readLine()) {
            if (s.indexOf("DO NOT EDIT STUFF BELOW: IT IS AUTOMATICALLY GENERATED") != -1 && s.indexOf("\"")==-1) skip = true;
            if (s.indexOf("DO NOT EDIT STUFF ABOVE: IT IS AUTOMATICALLY GENERATED") != -1 && s.indexOf("\"")==-1) break;
            if (!skip) out.append(s+"\n");
        }

        out.append("\n        // DO NOT EDIT STUFF BELOW: IT IS AUTOMATICALLY GENERATED\n");
        new Parser(MetaGrammar.make(), CharToken.top()).parse1(new CharToken.Stream(new InputStreamReader(new FileInputStream(args[0])))).toJava(out);
        out.append("\n        // DO NOT EDIT STUFF ABOVE: IT IS AUTOMATICALLY GENERATED\n");

        for(String s = br.readLine(); s != null; s = br.readLine()) out.append(s+"\n");
        br.close();

        OutputStream os = new FileOutputStream(fileName);
        PrintWriter p = new PrintWriter(new OutputStreamWriter(os));
        p.println(out.toString());
        p.flush();
        os.close();
    }

    public static final Tree meta =














        // DO NOT EDIT STUFF BELOW: IT IS AUTOMATICALLY GENERATED
new Tree(null, "gram", new Tree[] { new Tree(null, null, new Tree[] { }),
        new Tree(null, "grammar", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "s", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "=>", new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "g", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "m", new Tree[] { }),
        new Tree(null, "m", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "r", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "g", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "m", new Tree[] { })})})})})}),
        new Tree(null, "!::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "**", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { })})})})})})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "g", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "m", new Tree[] { }),
        new Tree(null, "m", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "r", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "=>", new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "+/", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "g", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "m", new Tree[] { }),
        new Tree(null, "m", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "r", new Tree[] { })})})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "/", new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, ":", new Tree[] { }),
        new Tree(null, ":", new Tree[] { }),
        new Tree(null, "=", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "a", new Tree[] { }),
        new Tree(null, "l", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "v", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, "/", new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "!", new Tree[] { }),
        new Tree(null, ":", new Tree[] { }),
        new Tree(null, ":", new Tree[] { }),
        new Tree(null, "=", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "a", new Tree[] { }),
        new Tree(null, "l", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "v", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "a", new Tree[] { }),
        new Tree(null, "l", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "v", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "+/", new Tree[] { new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "+/", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "c", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "j", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "|", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})})})})})})})})})})}),
        new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, ">", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})})})})})})})})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "c", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "j", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "s", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })})})})}),
        new Tree(null, "/", new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "s", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "&", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, "*/", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, "/", new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "s", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "&", new Tree[] { }),
        new Tree(null, "~", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, "*/", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "p", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "=>", new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "*/", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "p", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "=>", new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "?", new Tree[] { new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "+/", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})})})})})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "^", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})}),
        new Tree(null, "?", new Tree[] { new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "+/", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})})})})})})})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "p", new Tree[] { }),
        new Tree(null, "s", new Tree[] { }),
        new Tree(null, "2", new Tree[] { })})})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "p", new Tree[] { }),
        new Tree(null, "s", new Tree[] { }),
        new Tree(null, "x", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "p", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "/", new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "p", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "/", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "s", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "p", new Tree[] { }),
        new Tree(null, "s", new Tree[] { }),
        new Tree(null, "x", new Tree[] { })})})})}),
        new Tree(null, "/", new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "p", new Tree[] { }),
        new Tree(null, "s", new Tree[] { }),
        new Tree(null, "x", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "=", new Tree[] { }),
        new Tree(null, ">", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})})}),
        new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})})})})})})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "c", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "[~", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "range", new Tree[] { new Tree(null, "-", new Tree[] { })}),
        new Tree(null, "range", new Tree[] { new Tree(null, "]", new Tree[] { })}),
        new Tree(null, "range", new Tree[] { new Tree(null, "\\", new Tree[] { })}),
        new Tree(null, "range", new Tree[] { new Tree(null, "~", new Tree[] { })})})})})}),
        new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "p", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})})})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "g", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "=>", new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "c", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "g", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "c", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "-", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "c", new Tree[] { })})})})})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "=>", new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "n", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "T", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "m", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "l", new Tree[] { })})}),
        new Tree(null, "=>", new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "l", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "l", new Tree[] { })})}),
        new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { new Tree(null, "(", new Tree[] { }),
        new Tree(null, ")", new Tree[] { })}),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "/", new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { new Tree(null, "{", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "s", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "}", new Tree[] { })})})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { new Tree(null, "[", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, "*", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "g", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })})})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "]", new Tree[] { })})})})}),
        new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { new Tree(null, "[", new Tree[] { }),
        new Tree(null, "~", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, "*", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "g", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })})})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "]", new Tree[] { })})})})}),
        new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "*", new Tree[] { }),
        new Tree(null, "*", new Tree[] { })}),
        new Tree(null, null, new Tree[] { })})}),
        new Tree(null, null, new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "*", new Tree[] { })}),
        new Tree(null, null, new Tree[] { })})})})})})}),
        new Tree(null, "/", new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "*", new Tree[] { }),
        new Tree(null, "/", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "+", new Tree[] { }),
        new Tree(null, "+", new Tree[] { })}),
        new Tree(null, null, new Tree[] { })})}),
        new Tree(null, null, new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "+", new Tree[] { })}),
        new Tree(null, null, new Tree[] { })})})})})})}),
        new Tree(null, "/", new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "+", new Tree[] { }),
        new Tree(null, "/", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, "/", new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "?", new Tree[] { })}),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, "/", new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "(", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { new Tree(null, ")", new Tree[] { })}),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "/", new Tree[] { new Tree(null, "ps2", new Tree[] { new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { new Tree(null, "(", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "a", new Tree[] { }),
        new Tree(null, "l", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "v", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, ")", new Tree[] { })})})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})})})}),
        new Tree(null, "!::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, " ", new Tree[] { })})})})}),
        new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "/", new Tree[] { }),
        new Tree(null, "/", new Tree[] { })})}),
        new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "*", new Tree[] { new Tree(null, "[~", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "range", new Tree[] { new Tree(null, "\n", new Tree[] { })})})})})})})})})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\n", new Tree[] { })})})})}),
        new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\n", new Tree[] { })})})})}),
        new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\r", new Tree[] { })})})})})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "++", new Tree[] { new Tree(null, "[", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "-", new Tree[] { new Tree(null, "a", new Tree[] { }),
        new Tree(null, "z", new Tree[] { })}),
        new Tree(null, "-", new Tree[] { new Tree(null, "A", new Tree[] { }),
        new Tree(null, "Z", new Tree[] { })}),
        new Tree(null, "-", new Tree[] { new Tree(null, "0", new Tree[] { }),
        new Tree(null, "9", new Tree[] { })}),
        new Tree(null, "range", new Tree[] { new Tree(null, "_", new Tree[] { })})})})})})})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\"", new Tree[] { })})}),
        new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "+", new Tree[] { new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "[~", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "range", new Tree[] { new Tree(null, "\"", new Tree[] { })}),
        new Tree(null, "range", new Tree[] { new Tree(null, "\\", new Tree[] { })})})})})}),
        new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "p", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})})})})})})})})})})})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\"", new Tree[] { })})})})}),
        new Tree(null, "=>", new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\"", new Tree[] { }),
        new Tree(null, "\"", new Tree[] { })})})})}),
        new Tree(null, "", new Tree[] { })})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "p", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "=>", new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\\", new Tree[] { }),
        new Tree(null, "n", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "\n", new Tree[] { })})}),
        new Tree(null, "=>", new Tree[] { new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\\", new Tree[] { }),
        new Tree(null, "r", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { new Tree(null, "\r", new Tree[] { })})}),
        new Tree(null, "ps", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\\", new Tree[] { })})}),
        new Tree(null, "[~", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "range", new Tree[] { new Tree(null, "n", new Tree[] { })}),
        new Tree(null, "range", new Tree[] { new Tree(null, "r", new Tree[] { })})})})})})})})})})}),
        new Tree(null, null, new Tree[] { })})
        // DO NOT EDIT STUFF ABOVE: IT IS AUTOMATICALLY GENERATED
        ;
}








































